import 'dart:core';

class Validator {
  static const int passwordMin=6;
  static const int passwordMax=32;
  static final  RegExp passwordPattern=RegExp('(((?=.*\\d)(?=.*[a-z])(?=.*[A-Z])(?=.*[@#&%]).{'+'$passwordMin,$passwordMax}))');
  static final  RegExp numberPattern=RegExp(r'\d');
  static final  RegExp namePattern=RegExp(r"[a-zA-Z0-9 ]*");
  static final  RegExp specialCharactersPattern=RegExp(r"[$=\\\\|<>^*%]");
  static final RegExp emailPattern=RegExp(r'^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$');



  static bool isValidPassword(String password) {
    return true;
    //return password?.contains(passwordPattern, 0);
  }

  static bool isNameValid(String name) {
    return name!=null && namePattern.hasMatch(name);
  }

  static bool isEmailValid(String email) {
    return emailPattern.hasMatch(email);
  }

}

class KeyValueValidator {
  Map<String, String> _values = {};
  Map<String, String> _errors = {};

  addValue(String key, String value) {
    if (key != null) _values[key] = value;
  }

  addValues(Map<String, dynamic> keyValues) {
    if (keyValues != null) {
      keyValues.forEach((key,value) {
        if(value == null || value.runtimeType == String) {
          addValue(key,value);
        } else {
          removeValue(key);
        }
      });
    }
  }

  addError(String key, String error) {
    if (key != null) _errors[key] = error;
  }

  addErrors(Map<String, String> keyValues) {
    if (keyValues != null) _errors.addAll(keyValues);
  }

  removeValue(String key) {
    if (key != null) _values.remove(key);
  }

  removeError(String key) {
    if (key != null) _errors.remove(key);
  }

  clearValues() {
    _values.clear();
  }

  clearErrors() {
    _errors.clear();
  }

  bool containsValue(String key) {
    return _values.containsKey(key);
  }

  bool containsError(String key) {
    return _errors.containsKey(key);
  }

  String getValue(String key) {
    if (key == null) return null;
    return _values[key];
  }

  String getError(String key) {
    if (key == null) return null;
    return _errors[key];
  }

  int errorsLength() {
    return _errors.length;
  }

  int valuesLength() {
    return _values.length;
  }

  bool isEmpty(String str) {
    return str == null || str.trim().isEmpty;
  }

  bool checkEmpty(List<String> keys) {
    bool empty = true;
    if(keys!=null) {
      keys.forEach((key){
        if(isEmpty(getValue(key))) {
          empty = false;
          addError(key,  "Required field");
        }
      });
    }
    return empty;
  }

  bool checkEmail(String key) {
    if(!Validator.isEmailValid(getValue(key))) {
      addError(key,  "Invalid email");
      return false;
    }
    return true;
  }

  bool checkName(String key,[String errorMessage = "Invalid name"]) {
    if(!Validator.isNameValid(getValue(key))) {
      addError(key,  errorMessage);
      return false;
    }
    return true;
  }
}